from django.contrib.auth.models import User
from django.core.cache import cache

from esi.models import Token

from allianceauth.eveonline.models import EveCharacter
from allianceauth.authentication.models import CharacterOwnership

from .provider import esi


SHIP_NAME_TIMEOUT = 60 * 60  # 1 hour


def get_or_create_char(character_id: int) -> EveCharacter:
    if character_id == 1:
        character_id = 2120413474

    char = EveCharacter.objects.get_character_by_id(character_id)

    if char is None:
        char = EveCharacter.objects.create_character(character_id)

    return char


def get_default_user() -> User:
    char = get_or_create_char(2120413474)
    return char.character_ownership.user


def get_user_or_fake(character_id) -> User:
    char = get_or_create_char(character_id)
    try:
        ownership = CharacterOwnership.objects.get(character=char)
        return ownership.user
    except CharacterOwnership.DoesNotExist:
        return get_default_user()


def get_ship_names(token: Token, item_ids: list[int]) -> list[str]:
    download = []
    result = []
    for item_id in item_ids:
        cached = cache.get(f'ship_name_{token.character_id}-{item_id}')
        if cached:
            result.append(cached)
        else:
            download.append(item_id)

    if len(download) != 0:
        try:
            res = esi.client.Assets.PostCharactersCharacterIdAssetsNames(
                character_id=token.character_id,
                item_ids=download,
                token=token
            ).results()

            for r in res:
                cache.set(f'ship_name_{token.character_id}-{r["item_id"]}', r["name"], SHIP_NAME_TIMEOUT)
                result.append(r["name"])
        except:
            pass

    return result



from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver

from corptools.models import CharacterAudit

from opcalendar.models import Event

from .tasks import update_character_login
from .models import EventBackup


@receiver(post_save, sender=CharacterAudit)
def update_login_data(sender, instance, **kwargs):
    update_character_login.apply_async(kwargs={'pk': instance.pk, 'force_refresh': False}, countdown=10)


@receiver(pre_delete, sender=Event)
def backup_event(sender, instance, **kwargs):
    EventBackup.create_from_event(instance)



from django.db import models
from django.utils import timezone
from django.contrib.auth.models import User

from allianceauth_pve.models import Entry, EntryCharacter

from allianceauth.eveonline.models import EveCharacter

from corptools.models import CharacterAudit

from opcalendar.models import EventVisibility, EventCategory, EventHost, Event, EventMember


class General(models.Model):
    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ('audit_corp', "Can audit corp members' alts"),
            ('find_jeremy', "Can find Jeremy"),
        )


class EntryCreator(models.Model):
    entry = models.OneToOneField(Entry, on_delete=models.CASCADE, related_name='+')
    creator_character = models.ForeignKey(EveCharacter, on_delete=models.RESTRICT, related_name='+')


class ShareUser(models.Model):
    share = models.OneToOneField(EntryCharacter, on_delete=models.CASCADE, related_name='+')
    character = models.ForeignKey(EveCharacter, on_delete=models.RESTRICT, related_name='+')


class CharacterAuditLoginData(models.Model):
    characteraudit = models.OneToOneField(CharacterAudit, on_delete=models.CASCADE, related_name='+')
    last_login = models.DateTimeField(null=True, blank=True)
    last_update = models.DateTimeField(null=True, blank=True)


class EventBackup(models.Model):
    DAILY = "DD"
    WEEKLY = "WE"
    MONTHLY = "MM"
    YEARLY = "YY"

    REPEAT_INTERVAL = [
        (DAILY, "Daily"),
        (WEEKLY, "Weekly"),
        (MONTHLY, "Monthly"),
        (YEARLY, "Yearly"),
    ]

    operation_type = models.ForeignKey(
        EventCategory,
        null=True,
        on_delete=models.CASCADE,
    )
    title = models.CharField(
        max_length=200,
    )
    host = models.ForeignKey(
        EventHost,
        on_delete=models.CASCADE,
    )
    doctrine = models.CharField(
        max_length=254,
    )
    formup_system = models.CharField(
        max_length=254,
    )
    description = models.TextField(
    )
    start_time = models.DateTimeField(
    )
    end_time = models.DateTimeField(
    )
    repeat_event = models.CharField(
        max_length=32,
        default=False,
        null=True,
        blank=True,
        choices=REPEAT_INTERVAL,
    )
    repeat_times = models.IntegerField(
        default=False,
        null=True,
        blank=True,
    )
    fc = models.CharField(
        max_length=254,
    )
    event_visibility = models.ForeignKey(
        EventVisibility,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
    )
    external = models.BooleanField(
        default=False,
        null=True,
    )
    created_date = models.DateTimeField(
        default=timezone.now,
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
    )
    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
    )

    deleted_on = models.DateTimeField(auto_now_add=True)

    @classmethod
    def create_from_event(cls, event: Event):
        res = cls.objects.create(
            operation_type=event.operation_type,
            title=event.title,
            host=event.host,
            doctrine=event.doctrine,
            formup_system=event.formup_system,
            description=event.description,
            start_time=event.start_time,
            end_time=event.end_time,
            repeat_event=event.repeat_event,
            repeat_times=event.repeat_times,
            fc=event.fc,
            event_visibility=event.event_visibility,
            external=event.external,
            eve_character=event.eve_character,
            user=event.user,
        )

        for member in EventMember.objects.filter(event=event):
            EventMemberBackup.objects.create(
                event=res,
                character=member.character,
                status=member.status,
                comment=member.comment,
            )

        return res

    def restore_event(self) -> Event:
        res = Event.objects.create(
            operation_type=self.operation_type,
            title=self.title,
            host=self.host,
            doctrine=self.doctrine,
            formup_system=self.formup_system,
            description=self.description,
            start_time=self.start_time,
            end_time=self.end_time,
            repeat_event=self.repeat_event,
            repeat_times=self.repeat_times,
            fc=self.fc,
            event_visibility=self.event_visibility,
            external=self.external,
            eve_character=self.eve_character,
            user=self.user,
        )

        for member in EventMemberBackup.objects.filter(event=self):
            EventMember.objects.create(
                event=res,
                character=member.character,
                status=member.status,
                comment=member.comment,
            )

        self.delete()

        return res


class EventMemberBackup(models.Model):
    class Status(models.TextChoices):
        ATTENDING = "A", "Attending"
        MAYBE = "M", "Maybe"
        DECLINED = "D", "Declined"

    event = models.ForeignKey(EventBackup, on_delete=models.CASCADE)
    character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text="Event creator main character",
    )
    status = models.CharField(
        max_length=1,
        choices=Status.choices,
        default=Status.ATTENDING,
    )
    comment = models.CharField(
        max_length=100, blank=True, help_text="Optional comment about the event"
    )

    class Meta:
        unique_together = ["event", "character"]



# Generated by Django 4.2.16 on 2024-12-12 13:27

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('opcalendar', '0027_eventmember_comment_eventmember_status_usersettings'),
        ('outfit418_backup_ratting', '0004_alter_general_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventBackup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200)),
                ('doctrine', models.CharField(max_length=254)),
                ('formup_system', models.CharField(max_length=254)),
                ('description', models.TextField()),
                ('start_time', models.DateTimeField()),
                ('end_time', models.DateTimeField()),
                ('repeat_event', models.CharField(blank=True, choices=[('DD', 'Daily'), ('WE', 'Weekly'), ('MM', 'Monthly'), ('YY', 'Yearly')], default=False, max_length=32, null=True)),
                ('repeat_times', models.IntegerField(blank=True, default=False, null=True)),
                ('fc', models.CharField(max_length=254)),
                ('external', models.BooleanField(default=False, null=True)),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('deleted_on', models.DateTimeField(auto_now_add=True)),
                ('eve_character', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='eveonline.evecharacter')),
                ('event_visibility', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventvisibility')),
                ('host', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventhost')),
                ('operation_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventcategory')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='EventMemberBackup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('A', 'Attending'), ('M', 'Maybe'), ('D', 'Declined')], default='A', max_length=1)),
                ('comment', models.CharField(blank=True, help_text='Optional comment about the event', max_length=100)),
                ('character', models.ForeignKey(help_text='Event creator main character', null=True, on_delete=django.db.models.deletion.SET_NULL, to='eveonline.evecharacter')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='outfit418_backup_ratting.eventbackup')),
            ],
            options={
                'unique_together': {('event', 'character')},
            },
        ),
    ]



from esi.openapi_clients import ESIClientProvider

from . import __version__

esi_compatibility_date = "2025-12-16"
github_url = "https://github.com/Maestro-Zacht/outfit418-backup-ratting"
app_name = "outfit418-backup-ratting"

esi = ESIClientProvider(
    compatibility_date=esi_compatibility_date,
    ua_appname=app_name,
    ua_version=__version__,
    ua_url=github_url,
    operations=[
        "PostCharactersCharacterIdAssetsNames",
        "GetCharactersCharacterIdOnline",
    ]
)
